@@ -1,6 +1,5 @@
 package Errorlog;
 use Carp qw (cluck carp confess);
-our @CARP_NOT;
 use strict;
 use Time::HiRes qw( gettimeofday tv_interval);
 
@@ -14,42 +13,50 @@ use Sys::Hostname;
 use Sys::Syslog;
 use Errorlog::TeeOutput;
 
-my $version = '$Id: Errorlog.pm 43121 2015-03-25 14:05:31Z anton $ ';
-our $VERSION = "1.41";
+$Carp::Internal{ (__PACKAGE__) }++;
+#$Carp::Internal{ (__Errorlog__) }++;
+
+
+my $version = '$Id: Errorlog.pm 43775 2015-08-26 07:48:11Z anton $ ';
+our $VERSION = "1.43";
 
 #global variable, needed for TeeOutput
 our $SELF;
 
+
+#  Geef een stackdump bij het sterven.
 sub import {
 	$SIG{__DIE__} = sub { 
 		die @_ if ref $_[0];
 		if ($SELF->{fields}->{autoconfess}) {
 			local $_ = join('', @_);
-			if (s/^.*\K at \S+ line \d+\.?\n\z//) {
-				local $Carp::CarpLevel = 1;
-				confess($_);
-			} else {
-				die $_;
-			}
+			local $Carp::CarpLevel = 1;
+			confess("die handler in Errorlog: ", @_);
 		}
 	}
 }
 
 sub end {
-	carp  "Errorlog:end; Exitcode is: $?"  if $Baseobject::verbosity >=2;
-	if ($SELF->{fields}->{mailonfailure}){
-		my $logbuffer = 'no logbuffer'; 
-		$logbuffer = $SELF->{fields}->{logbuffer} if $SELF->{fields}->{logbuffer};
-		$SELF->mail("Errorlog v$VERSION failure: Errorstatus: $?, \n$logbuffer") ;
+	my $exitCode = $?;
+	warn "Errorlog:end; Exitcode is: $exitCode" if $Baseobject::verbosity >=2;
+	
+	if ($exitCode) {
+		if ($SELF->{fields}->{mailonfailure}){
+			my $logbuffer = 'no logbuffer'; 
+			$logbuffer = $SELF->{fields}->{logbuffer} if $SELF->{fields}->{logbuffer};
+			$SELF->mail("Errorlog $VERSION failure: Errorstatus: $?, \n$logbuffer") ;
+		}
+		carp('died');
 	}
 }
 
 END {
-	local $? = $?;
+	my $exitCode = $?;
 	return if $SELF->{fields}->{skipEND};
-	if ($?) {
+	if ($exitCode) {
 		# call an extra 'end' method to be able to call this method from an eternally living
 		# perl interpretor such as modperl.
+		local $? = $exitCode;
 		end();
 	}
 }
@@ -64,12 +71,13 @@ sub new {
 	my $fh;
 	my $logfile = $self->makelogfilename();
 
+	# moet worden $LOCALSTATEDIR/$PACKAGE/log
 	my $logdir = $self->{LOCALSTATEDIR};
 	my $configdir = $self->{SYSCONFDIR};
 
 	my $hostname = $self->{hostname} = hostname();
 	my $user = $ENV{'REMOTE_USER'};
-	$user = $ENV{'USER'} unless (defined ($user));
+	$user = $ENV{'USER'} unless defined ($user);
 
 	$self->allows(
 		{
@@ -126,14 +134,32 @@ sub new {
 	$logdir = $self->field('logdir');
 	$logfile = $self->field('logfile');
 
+#	warn "Errorlog: logdir: $logdir, verbosity: $Baseobject::verbosity ";
+
 	if ($self->{fields}->{noDebug2Syslog}) {
 		if ($Baseobject::verbosity >= 2) {
 			if ($self->{outputmodes}->{syslog}) {
-				$syslogSuppressed = "Warning ($0): Debug output is not allowed in syslog but redirected to: \"$logdir/$logfile\"\n";
-				warn $syslogSuppressed;
-				$self->{outputmodes}->{logfile} = 1;
-				syslog('debug', $syslogSuppressed);
-				$self->{outputmodes}->{syslog} = 0;
+
+				# Call openlog() in order to write the message that writing messages to syslog is not allowed... 
+				# If this openlog() call is omitted syslog() messages end up in user.log
+
+				my $facility = $self->{fields}->{facility};
+				if (openlog($0, 'cons,pid', $facility) ) {
+					$syslogSuppressed = "Warning ($0): Debug output is not allowed bij Errorlog ($VERSION) in syslog (facility: $facility) but redirected to: \"$logdir/$logfile\"\n";
+
+					syslog('debug', $syslogSuppressed);
+					closelog();
+
+					# waar komt dit terecht
+					warn "warn: $syslogSuppressed";
+					
+					$self->{outputmodes}->{logfile} = 1;
+					$self->{outputmodes}->{syslog} = 0;
+					
+				} else {
+					$self->error("Could not open syslog");
+				}
+
 			}
 		}
 	}
@@ -196,8 +222,8 @@ sub new {
 	$SELF=$self;
 
 	openTee (*STDERR, @params, \%params);
-	carp "Errorlog (Version: $VERSION) Entering logmode for: $0 " if $Baseobject::verbosity >=2;
-	carp ("configdir: $self->{fields}->{configdir}") if $Baseobject::verbosity >=2 ;
+	warn "Errorlog (Version: $VERSION) Entering logmode for: $0 " if $Baseobject::verbosity >=2;
+	warn ("configdir: $self->{fields}->{configdir}") if $Baseobject::verbosity >=2 ;
 
 	return $self;
 }
@@ -404,11 +430,11 @@ sub banner {
 sub mail {
 	my $self = shift;
 	my $string = join("\n",@_);
-	carp "Sending mail to: $self->{fields}->{mailto}" ;
-	carp "using mailprog: $self->{fields}->{mailprogram}" if $Baseobject::verbosity >=2 ;
+	warn "Sending mail to: $self->{fields}->{mailto}" ;
+	warn "using mailprog: $self->{fields}->{mailprogram}" if $Baseobject::verbosity >=2 ;
 
 	unless ($self->{fields}->{mailto}) {
-		carp "ERROR: No mail send, 'mailto' not set!";
+		warn "ERROR: No mail send, 'mailto' not set!";
 		return;
 	}
 
@@ -501,10 +527,10 @@ sub overrule {
 		foreach $basepart ('defaults', $self->basefilename(), 'overrule') {
 			$configfile = "$configdir/${basepart}_errorlog.cf";
 			next unless -r $configfile;
-			carp "Using additional configfile: $configfile" if $Baseobject::verbosity >= 2 ;
+			warn "Using additional configfile: $configfile" if $Baseobject::verbosity >= 2 ;
 
 			unless (open ($fh, $configfile)) {
-				carp "Found but could not open configfile: $configfile, $!";
+				warn "Found but could not open configfile: $configfile, $!";
 				return;
 			}
 
@@ -546,14 +572,14 @@ sub overrule {
 				elsif  ($ref eq '') {
 					$self->field($name,$value);
 				} else {
-					carp "ignoring variable: $name: type \"$ref\" not supported";
+					warn "ignoring variable: $name: type \"$ref\" not supported";
 				}
 			}
 			warn "Errorlog (version: $VERSION): Using configfile: $configfile" 
 				if $Baseobject::verbosity > 1 or $self->error();
 		}
 	} else {
-		carp "Warning; Errorlog (version: $VERSION) configdir: \"$configdir\" does not exist";
+		warn "Warning; Errorlog (version: $VERSION) configdir: \"$configdir\" does not exist";
 	}
 }
 
